"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeUrl = exports.getEnv = exports.defaultDevices = void 0;
exports.generateDynamicTestProjects = generateDynamicTestProjects;
exports.convertDeviceToPlaywrightProject = convertDeviceToPlaywrightProject;
exports.matchUrlHostToEnv = matchUrlHostToEnv;
exports.matchHostnameToEnv = matchHostnameToEnv;
exports.deepMerge = deepMerge;
exports.isObject = isObject;
const test_1 = require("@playwright/test");
/**
 * Consts
 */
// What default devices should we test on
exports.defaultDevices = ['Desktop Edge', 'Desktop Safari', 'iPhone 15'];
// Get the current environment
const getEnv = () => process.env.npm_config_env ||
    process.env.PLAYWRIGHT_ENV ||
    process.env.ENV ||
    'local';
exports.getEnv = getEnv;
const normalizeUrl = (url) => url.endsWith('/') ? url.slice(0, -1) : url;
exports.normalizeUrl = normalizeUrl;
/**
 * Functions
 */
function generateDynamicTestProjects(projects) {
    const firstMobile = projects.find(name => test_1.devices[name] ? test_1.devices[name].isMobile : undefined);
    const firstDesktop = projects.find(name => test_1.devices[name] ? !test_1.devices[name].isMobile : undefined);
    return projects.map((device, index) => convertDeviceToPlaywrightProject(device, {
        firstDevice: (index === 0),
        firstMobile: (device === firstMobile),
        firstDesktop: (device === firstDesktop),
    }));
}
// Convert a single device name to a Playwright project
function convertDeviceToPlaywrightProject(title = '', config) {
    const deviceConfig = test_1.devices[title];
    if (!deviceConfig) {
        throw new Error(`Device configuration not found: ${title}`);
    }
    // Create an item for the device
    const firstDeviceTest = '|unit|spec', item = {
        name: title,
        use: deviceConfig,
        testMatch: []
    };
    // If the first device, match filename.unit.ts & filename.spec.ts
    item.testMatch.push('test' + (config?.firstDevice ? firstDeviceTest : ''));
    // If the device is mobile, add `filename.mobile.test.ts
    item.testMatch.push(deviceConfig.isMobile ?
        'mobile\\.(test' + (config?.firstMobile ? firstDeviceTest : '') + ')'
        : '');
    // If the device is mobile, add `filename.mobile.test.ts
    item.testMatch.push(!deviceConfig.isMobile ?
        'desktop\\.(test' + (config?.firstDesktop ? firstDeviceTest : '') + ')'
        : '');
    item.testMatch = item.testMatch
        .filter((match) => match)
        .map((match) => new RegExp(`^[^.]+\\.(${match})\\.[cm]?[jt]sx?$`));
    // Return the item
    return item;
}
// Swap a host based on environment
function matchUrlHostToEnv(hosts, path) {
    // Is the path a valid URL
    if (!URL.canParse(path)) {
        return path;
    }
    // Parse the URL
    const url = new URL(path);
    // Get the new origin
    const base = matchHostnameToEnv(hosts, (0, exports.normalizeUrl)(url.origin));
    // Reconstruct the URL with the new base
    return (0, exports.normalizeUrl)(base) + url.pathname + url.search + url.hash;
}
// Swap the host based on environment
function matchHostnameToEnv(hosts, origin) {
    // Get the current environment
    const env = (0, exports.getEnv)();
    // Loop through hosts
    for (const host of hosts) {
        // Get all the URLs from the host
        const urls = new Set(Object.values(host).map(url => (0, exports.normalizeUrl)(url)));
        // Is our origin in that set?
        if (urls.has(origin)) {
            // Find the matching key without looping through all entries
            for (const key in host) {
                if ((0, exports.normalizeUrl)(host[key]) === origin) {
                    return key === env ? origin : host[env] || origin;
                }
            }
        }
    }
    return origin;
}
// Allow deep merging of objects
function deepMerge(target, ...sources) {
    if (!sources.length)
        return target;
    const source = sources.shift();
    if (isObject(target) && isObject(source)) {
        for (const key in source) {
            if (isObject(source[key])) {
                if (!target[key])
                    Object.assign(target, { [key]: {} });
                deepMerge(target[key], source[key]);
            }
            else {
                Object.assign(target, { [key]: source[key] });
            }
        }
    }
    return deepMerge(target, ...sources);
}
// Detect if a variable is an object
function isObject(item) {
    return (item && typeof item === 'object' && !Array.isArray(item));
}
//# sourceMappingURL=utils.js.map